/*
 * Common resource server functions.
 *
 * Copyright (C) 2018, 2019, 2020, 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <fsserver/resource.h>
#include <ipc/server.h>



/* Convenience abstraction for blocking servers. */

class ResourceServer
{
protected:
  Resource *_resource;
  ipc_server_config_type *_config;

public:
  explicit ResourceServer(Resource *resource)
  : _resource(resource)
  {
    _config = new ipc_server_config_type;
    ipc_server_init_config(_config);
  }

  /* Access to configuration. */

  ipc_server_config_type *config()
  { return _config; }

  /* Server IPC gate allocation. */

  long bind(const char *name);

  /* Server initiation. */

  long start();

  long start_thread();

  long start_thread(l4_cap_idx_t *server);
};



/* Server initialisation. */

void resource_init_config(ipc_server_config_type *config, Resource *resource);

void resource_set_config_threaded(ipc_server_config_type *config,
                                  l4_cap_idx_t thread, int new_thread);

/* Server initiation. */

long resource_start_config(ipc_server_config_type *config, Resource *resource);

/* Server finalisation. */

void resource_thread_finaliser(ipc_server_config_type *config);

// vim: tabstop=2 expandtab shiftwidth=2
